package com.cnbm.packing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.cnbm.admin.utils.BaseSupportUtils;
import com.cnbm.common.page.PageData;
import com.cnbm.common.service.impl.CrudServiceImpl;
import com.cnbm.common.utils.ConvertUtils;
import com.cnbm.packing.dto.IdVo;
import com.cnbm.packing.dto.PowerReportDTO;
import com.cnbm.packing.dto.WoPackagingBoxDTO;
import com.cnbm.packing.entity.WoPackagingBox;
import com.cnbm.packing.entity.WoPackagingBoxSubstrate;
import com.cnbm.packing.mapper.WoPackagingBoxMapper;
import com.cnbm.packing.mapper.WoPackagingBoxSubstrateMapper;
import com.cnbm.packing.param.PowerReportQueryParam;
import com.cnbm.packing.service.WoPackagingBoxServiceBiz;
import com.cnbm.packing.service.WoPackagingBoxSubstrateServiceBiz;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 包装箱表
 *
 * @author codeGenerator
 * @since 2023-02-15
 */
@Service
public class WoPackagingBoxServiceBizImpl extends CrudServiceImpl<WoPackagingBoxMapper, WoPackagingBox, WoPackagingBoxDTO> implements WoPackagingBoxServiceBiz {

    @Autowired
    private WoPackagingBoxMapper mapper;
    @Autowired
    private WoPackagingBoxSubstrateMapper substrateMapper;
    @Autowired
    private WoPackagingBoxSubstrateServiceBiz woPackagingBoxSubstrateServiceBiz;


    @Override
    public QueryWrapper<WoPackagingBox> getWrapper(Map<String, Object> params){

        QueryWrapper<WoPackagingBox> wrapper = new QueryWrapper<>();

        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        if(params.get("startTime")!=null && params.get("endTime")!=null){
            LocalDateTime startTime = LocalDateTime.parse(params.get("startTime").toString(),df);
            LocalDateTime endTime = LocalDateTime.parse(params.get("endTime").toString(),df);
            wrapper.between(startTime!=null && endTime!=null, WoPackagingBox.CREATE_TIME, startTime, endTime);
        }
        String boxNo = (String) params.get("boxNo");
        wrapper.like(StringUtils.isNotBlank(boxNo), WoPackagingBox.BOX_NO, boxNo);
        if(params.get("lineBody")!=null) {
            Integer lineBody = Integer.parseInt(params.get("lineBody").toString());
            wrapper.eq(ObjectUtils.isNotNull(lineBody), WoPackagingBox.LINE_BODY, lineBody);
        }
        if(params.get("printStatus")!=null) {
            Integer printStatus = Integer.parseInt(params.get("printStatus").toString());
            wrapper.eq(ObjectUtils.isNotNull(printStatus), WoPackagingBox.PRINT_STATUS, printStatus);
        }
        if(params.get("model")!=null) {
            Integer model = Integer.parseInt(params.get("model").toString());
            wrapper.eq(ObjectUtils.isNotNull(model), WoPackagingBox.MODEL, model);
        }
        if(params.get("startTime")==null && params.get("endTime")==null && params.get("boxNo")==null && params.get("lineBody")==null && params.get("printStatus")==null){
            wrapper.apply(true, "TO_DAYS(NOW())-TO_DAYS(create_time) = 0");
        }

        return wrapper;
    }

    @Override
    public PageData<WoPackagingBoxDTO> page (Map<String, Object> params){

        IPage<WoPackagingBox> page = baseDao.selectPage(
                getPage(params, WoPackagingBox.CREATE_TIME, false),
                getWrapper(params)
        );
        return getPageData(page, WoPackagingBoxDTO.class);
    }

    @Override
    public WoPackagingBoxDTO get(Long id) {
        WoPackagingBox entity = baseDao.selectById(id);
        return ConvertUtils.sourceToTarget(entity, WoPackagingBoxDTO.class);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public IdVo add(WoPackagingBoxDTO dto) {
        WoPackagingBox entity = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        BaseSupportUtils.setCommonField(entity);
        insert(entity);
        return IdVo.builder().id(entity.getId()).build();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(WoPackagingBoxDTO dto) {
        WoPackagingBox entity = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        BaseSupportUtils.setUpdateCommonField(entity);
        updateById(entity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long[] ids) {
        //删除
        deleteBatchIds(Arrays.asList(ids));
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<WoPackagingBoxDTO> list() {
        List<WoPackagingBoxDTO> list = mapper.list();
        return list;
    }

    @Override
    public WoPackagingBoxDTO printList(Long id) {
        WoPackagingBox entity = baseDao.selectById(id);
        WoPackagingBoxDTO dto = ConvertUtils.sourceToTarget(entity, WoPackagingBoxDTO.class);
        QueryWrapper<WoPackagingBoxSubstrate> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(entity.getBoxNo()), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, entity.getBoxNo());
        List<WoPackagingBoxSubstrate> woPackagingBoxSubstrateList = substrateMapper.selectList(wrapper);
        dto.setSubstrateList(woPackagingBoxSubstrateList);
        return dto;
    }

    @Override
    public List<WoPackagingBoxSubstrate> substrateList(Map<String, Object> params) {
        QueryWrapper<WoPackagingBox> boxQueryWrapper = getWrapper(params);
        List<WoPackagingBox> boxList = mapper.selectList(boxQueryWrapper);
        List<WoPackagingBoxSubstrate> substrateList = new ArrayList<>();
        for(WoPackagingBox box: boxList){
            QueryWrapper<WoPackagingBoxSubstrate> substrateQueryWrapper = new QueryWrapper<>();
            substrateQueryWrapper.eq(StringUtils.isNotBlank(box.getBoxNo()), WoPackagingBoxSubstrate.PACKAGING_BOX_ID, box.getBoxNo());
            List<WoPackagingBoxSubstrate> woPackagingBoxSubstrateList = substrateMapper.selectList(substrateQueryWrapper);
            substrateList.addAll(woPackagingBoxSubstrateList);
        }
        return substrateList;
    }

    @Override
    public List<WoPackagingBox> boxList(String woSubstrateId) {
        QueryWrapper<WoPackagingBoxSubstrate> substrateQueryWrapper = new QueryWrapper<>();
        substrateQueryWrapper.eq(StringUtils.isNotBlank(woSubstrateId), WoPackagingBoxSubstrate.WO_SUBSTRATE_ID, woSubstrateId);
        List<WoPackagingBoxSubstrate> substrateList = substrateMapper.selectList(substrateQueryWrapper);
        List<WoPackagingBox> boxList = new ArrayList<>();
        for(WoPackagingBoxSubstrate substrate: substrateList){
            QueryWrapper<WoPackagingBox> boxQueryWrapper = new QueryWrapper<>();
            boxQueryWrapper.like(StringUtils.isNotBlank(substrate.getPackagingBoxId()), WoPackagingBox.BOX_NO, substrate.getPackagingBoxId());
            List<WoPackagingBox> woPackagingBoxList = mapper.selectList(boxQueryWrapper);
            boxList.addAll(woPackagingBoxList);
        }
        return boxList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public IdVo insertSubstrateManual(WoPackagingBoxDTO dto) {
        //新增包装箱
        WoPackagingBox box = ConvertUtils.sourceToTarget(dto, WoPackagingBox.class);
        BaseSupportUtils.setCommonField(box);
        insert(box);
        //新增模组
        List<WoPackagingBoxSubstrate> substrateList = dto.getSubstrateList();
        for(WoPackagingBoxSubstrate substrate : substrateList){
            substrate.setPackagingBoxId(box.getBoxNo());
            substrate.setPowerLevel(box.getPowerLevel());
            BaseSupportUtils.setCommonField(substrate);
            woPackagingBoxSubstrateServiceBiz.insert(substrate);
        }
        return IdVo.builder().id(box.getId()).build();
    }

    @Override
    public int updateIsArrivedByBoxNo(Integer isArrived, String boxNo) {
        return mapper.updateIsArrivedByBoxNo(isArrived,boxNo);
    }

    @Override
    public List<PowerReportDTO> powerReport(PowerReportQueryParam queryParam) {
        return substrateMapper.powerReport(queryParam);
    }

    @Override
    public List<String> orderNameList() {
        return substrateMapper.orderNameList();
    }
}